package cn.com.qmth.stmms.admin.thread;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.stmms.admin.utils.HttpUtil;
import cn.com.qmth.stmms.admin.utils.UpyunConfig;
import cn.com.qmth.stmms.biz.exam.model.DataSync;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.service.DataSyncService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.common.enums.LockType;
import cn.com.qmth.stmms.common.utils.PictureUrlBuilder;

public class DataSyncThread implements Runnable {

    protected static Logger log = LoggerFactory.getLogger(DataSyncThread.class);

    private DataSync dataSync;

    private ExamStudentService studentService;

    private ExamSubjectService subjectService;

    private ExamService examService;

    private LockService lockService;

    private DataSyncService dataSyncService;

    private Integer pageSize;

    private String baseDir;

    private UpyunConfig config;

    private HttpUtil subjectHttp;

    private HttpUtil studentHttp;

    public static final String PAPER_STRUCT = "paperStructList";

    public static final String SUBJECT_CODE = "subjectCode";

    public static final String SUBJECT_NAME = "subjectName";

    public static final String DATA_LIST = "dataList";

    public static final String START_ID = "startId";

    public static final String NEXT_ID = "nextId";

    public static final String DEFAULT_NULL = "#";

    public static final String SIZE = "size";

    public DataSyncThread(DataSync dataSync, Integer pageSize, String baseDir, UpyunConfig config,
            LockService lockService, DataSyncService dataSyncService, ExamService examService,
            ExamStudentService studentService, ExamSubjectService subjectService) {
        this.dataSync = dataSync;
        this.lockService = lockService;
        this.dataSyncService = dataSyncService;
        this.examService = examService;
        this.studentService = studentService;
        this.subjectService = subjectService;
        this.pageSize = pageSize;
        this.baseDir = baseDir;
        this.config = config;
        this.subjectHttp = new HttpUtil(dataSync.getSubjectUrl(), dataSync.getSecretKey(), dataSync.getAppId(),
                dataSync.getRootOrgId());
        this.studentHttp = new HttpUtil(dataSync.getStudentUrl(), dataSync.getSecretKey(), dataSync.getAppId(),
                dataSync.getRootOrgId());
    }

    @Override
    public void run() {
        log.info("start data sync for examId=" + dataSync.getExamId());
        try {
            // 获取考试信息
            Exam exam = examService.findById(dataSync.getExamId());
            dataSync.setSchoolId(exam.getSchoolId());
            DataSync sync = dataSyncService.findByExamId(dataSync.getExamId());
            if (sync == null) {
                sync = dataSync;
                sync.setCreateTime(new Date());
                dataSyncService.save(sync);
            }
            JSONObject datas = new JSONObject();
            datas.accumulate("examId", sync.getCloudExamId());
            String subjectResult = subjectHttp.httpAction(null, datas.toString());
            JSONObject subjectJson = JSONObject.fromObject(subjectResult);
            JSONArray subjectArray = subjectJson.getJSONArray(PAPER_STRUCT);
            for (int i = 0; i < subjectArray.size(); i++) {
                JSONObject subject = subjectArray.getJSONObject(i);
                String subjectCode = subject.getString(SUBJECT_CODE);
                String subjectName = subject.getString(SUBJECT_NAME);
                Long startId = 0L;
                if (null != sync.getNextId()) {
                    startId = sync.getNextId();
                }
                ExamSubject examSubject = subjectService.find(exam.getId(), subjectCode);
                if (examSubject != null && !subjectCode.equals(sync.getSubjectCode())) {
                    continue;
                }
                while (startId != null) {
                    datas.put(SUBJECT_CODE, subjectCode);
                    datas.put(START_ID, startId);
                    datas.put(SIZE, pageSize);
                    String studentResult = studentHttp.httpAction(null, datas.toString());

                    JSONObject studentJson = JSONObject.fromObject(studentResult);

                    Long nextId = studentJson.getLong(NEXT_ID);
                    if (startId - nextId == 0) {
                        startId = null;
                    } else {
                        startId = nextId;
                    }
                    Object obj = studentJson.get(DATA_LIST);
                    if (obj instanceof JSONObject && ((JSONObject) obj).isNullObject()) {
                        continue;
                    }
                    JSONArray studentArray = studentJson.getJSONArray(DATA_LIST);
                    List<ExamStudent> list = new ArrayList<ExamStudent>();
                    Date now = new Date();
                    for (int j = 0; j < studentArray.size(); j++) {
                        JSONObject student = studentArray.getJSONObject(j);
                        ExamStudent examStudent = new ExamStudent();
                        examStudent.setExamId(sync.getExamId());

                        examStudent.setStudentCode(student.getString("studentCode"));
                        examStudent.setName(student.getString("name"));
                        examStudent.setCollege(DEFAULT_NULL);
                        examStudent.setClassName(DEFAULT_NULL);
                        examStudent.setTeacher(DEFAULT_NULL);
                        examStudent.setCampusName(DEFAULT_NULL);
                        examStudent.setSubjectCode(subjectCode);
                        examStudent.setSubjectName(subjectName);
                        examStudent.setExamNumber(student.getString("examNumber"));
                        examStudent.setPaperType(null);

                        examStudent.setSchoolId(exam.getSchoolId());
                        examStudent.setAbsent(false);
                        examStudent.setUpload(true);
                        examStudent.setManualAbsent(false);
                        examStudent.setBreach(false);
                        examStudent.setException(false);
                        examStudent.setSliceCount(0);
                        examStudent.setSheetCount(0);
                        examStudent.setObjectiveScore(0d);
                        examStudent.setSubjectiveScore(0d);
                        examStudent.setAnswers(null);
                        examStudent.setBatchCode(null);
                        examStudent.setUploadTime(now);
                        list.add(examStudent);

                        String answerJson = student.getString("subjectives");
                        File file = new File(new File(baseDir, config.getMediaBucket()),
                                PictureUrlBuilder.getAnswerJson(exam.getId(), subjectCode, null,
                                        examStudent.getExamNumber()));
                        if (!file.exists()) {
                            file.getParentFile().mkdirs();
                            file.createNewFile();
                        }
                        FileOutputStream fos = new FileOutputStream(file);
                        fos.write(new byte[] { (byte) 0xEF, (byte) 0xBB, (byte) 0xBF });
                        BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(fos, "UTF-8"));
                        bw.write(answerJson);
                        bw.close();
                    }
                    int count = studentService.batchSave(list);
                    if (count > 0) {
                        subjectService.updateUploadCount(exam.getId(), subjectCode,
                                (int) studentService.countUploadedByExamIdAndSubjectCode(exam.getId(), subjectCode));
                    }
                    sync.setUpdateTime(new Date());
                    sync.setNextId(startId);
                    sync.setSubjectCode(subjectCode);
                    dataSyncService.save(sync);
                }
            }
            sync.setFinished(true);
            dataSyncService.save(sync);
        } catch (Exception e) {
            log.error("data sync exception for examId=" + dataSync.getExamId(), e);
        } finally {
            lockService.unlock(LockType.DATA_SYNC, dataSync.getExamId());
            log.info("finish data sync for examId=" + dataSync.getExamId());
        }
    }
}
